/* This is based on the work outlined in the statement below.
 * As such it is released under the same license and conditions.
 * No warranty is provided... blah blah
 * Date of modification/creation 2009-2010
 * Author (in part): Brian Starkey
 * 
 * Parts also based on http://eeegadgets.blogspot.com/2009/07/usb-mail-notifier-final-revision.html
 * 
 * Basically runs ifconfig every second and deduces the current bit-rate, then decides on the color and sends it to the USB device
 * 
 */
/* Project: hid-custom-rq example
 * Author: Christian Starkjohann
 * Creation Date: 2008-04-07
 * Tabsize: 4
 * Copyright: (c) 2008 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt), GNU GPL v3 or proprietary (CommercialLicense.txt)
 * This Revision: $Id: main.c 692 2008-11-07 15:07:40Z cs $
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <usb.h>        /* this is libusb */
#include <math.h>
#include <time.h>
#include <sys/time.h>
#define PI 3.14159265358979323846
#define DEG2RAD(DEG) ((DEG)*((PI)/(180.0)))
#include "opendevice.h" /* common code moved to separate module */

#include "requests.h"   /* custom request numbers */
#include "usbconfig.h"  /* device's VID/PID and names */

void display(unsigned char * data, usb_dev_handle * handle);

int main(int argc, char ** argv) {
	FILE *fp;
	unsigned long long int temp = 0;
	unsigned long long int old = 0;
	int kBps = 0;
	int debug = 0;
	char command[100];
	char output[20];
	int inactive = 0;
	usb_dev_handle      *handle = NULL;
	const unsigned char rawVid[2] = {USB_CFG_VENDOR_ID}; 
	const unsigned char rawPid[2] = {USB_CFG_DEVICE_ID};
	char vendor[] = {USB_CFG_VENDOR_NAME, 0};
	char product[] = {USB_CFG_DEVICE_NAME, 0};
	unsigned char data[] = {0,0,0,8};
	double t1 = 0, t2 = 0;
	struct timeval tim;
	int vid, pid;
  	
	if (argc > 1) {
		sprintf( command, "ifconfig %s", argv[1]);
		if ( system(command)) {
			return 1;
		}
		if ( argc == 3) {
			debug = 1;
		}
	}
	else {
		puts("Device is required.");
		puts("Usage: play [device]");		
		puts("Where device is the path to the network device to monitor (/dev/eth0 for instance)");
		return 1;
	}


  	usb_init();
  	/* compute VID/PID from usbconfig.h so that there is a central source of information */
    vid = rawVid[1] * 256 + rawVid[0];
    pid = rawPid[1] * 256 + rawPid[0];
    /* The following function is in opendevice.c: */
    
    if(usbOpenDevice(&handle, vid, vendor, pid, product, NULL, NULL, NULL) != 0){
        fprintf(stderr, "Could not find USB device \"%s\" with vid=0x%x pid=0x%x\n", product, vid, pid);
        exit(1);
    }

	sprintf( command, "ifconfig %s | grep \"RX bytes\" | awk -F: '{print $2}' | awk -F' ' '{print $1}'", argv[1]);
	//printf("Command is: %s", command);
	
	while (1) {
		old = temp;
		fp = popen(command, "r" );
		while ( fgets( output, sizeof output, fp ) != NULL ) {
  			sscanf( output, "%lli", &temp);
		}
		pclose( fp );
		
        gettimeofday(&tim, NULL);
        t2 = t1;
        t1=tim.tv_sec+(tim.tv_usec/1000000.0);
        t2 = (t1-t2);
        //printf("t2: %f\n", t2);
		if (t2 < 2) {
			kBps = ((temp - old)/t2)/1024;
		}
		
			if (debug) printf("KBytes: %3i, R:%3i, G:%3i, B:%3i\n", kBps, data[0], data[1], data[2]);
			if (kBps < 10) {
				if (inactive <= 85) inactive+=5;
				else if (inactive < 99) inactive++;
			}
			else {
				inactive = 0;
			}
			if (kBps < 60) {
				if (inactive) {
					data[0] = 100 - inactive;
					data[1] = 0;
					data[2] = 0;
				}
				else {
					data[0] = 150;
					data[1] = (unsigned char)(510*(1-cos(DEG2RAD(kBps))));
					data[2] = 0;
				}
			}
			else if (kBps < 120) {
				data[0] = 150-(unsigned char)(300*(1-cos(DEG2RAD(kBps-60))));
				data[1] = 255;
				data[2] = 0;
			}
			else if (kBps < 180) {
				data[0] = 0;
				data[1] = 255;
				data[2] = (unsigned char)(510*(1-cos(DEG2RAD(kBps-120))));
			}
			else if (kBps < 240) {
				data[0] = 0;
				data[1] = 255-(unsigned char)(510*(1-cos(DEG2RAD(kBps-180))));
				data[2] = 255;
			}
			else if (kBps < 300) {
				data[0] = (unsigned char)(300*(1-cos(DEG2RAD(kBps-240))));
				data[1] = 0;
				data[2] = 255;
			}
			else if (kBps < 360) {
				data[0] = 150;
				data[1] = (unsigned char)(510*(1-cos(DEG2RAD(kBps-300))));
				data[2] = 255;
			}
			else {
				data[0] = 150;
				data[1] = 255;
				data[2] = 255;
			}
		
		//wait = time(NULL);
		display(data, handle);
		if (system("sleep 1s")) {
			data[0] = 0;
			data[1] = 0;
			data[2] = 0;
			display(data, handle);
			return 0;
		}
	}
}


void display(unsigned char * data, usb_dev_handle * handle) {
	unsigned char rVal = data[0];
	unsigned char gVal = data[1];
	unsigned char bVal = data[2];
	int speed = data[3];
	int cnt;
	int wV = gVal << 8;			// wValue.bytes[1] = green
	int wI = speed << 8;			// wIndex.bytes[1] = speed
	char buffer[4];
	wV = wV | rVal;					// wValue.bytes[0] = red
	wI = wI | bVal;					// wIndex.bytes[0] = blue
	
	cnt = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_OUT, CUSTOM_RQ_SET_COLOUR, wV, wI, buffer, 0, 5000);
}
